
/******************************************************************************
* DISCLAIMER

* This software is supplied by Renesas Electronics Corporation and is only 
* intended for use with Renesas products. No other uses are authorized.

* This software is owned by Renesas Electronics Corporation and is protected under 
* all applicable laws, including copyright laws.

* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES 
* REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, 
* INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
* PARTICULAR PURPOSE AND NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY 
* DISCLAIMED.

* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS 
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE 
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES 
* FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS 
* AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

* Renesas reserves the right, without notice, to make changes to this 
* software and to discontinue the availability of this software.
* By using this software, you agree to the additional terms and 
* conditions found by accessing the following link:
* http://www.renesas.com/disclaimer

******************************************************************************/

/******************************************************************************
* Copyright (C) 2012 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************
* File Name    : userif.c
* Version      : 2.0
* Device(s)    : RL78G14
* Tool-Chain   : RL78 IAR Embedded workbench V 1.20.x
* H/W Platform : YRMCKITRL78G14 V2.0
* Description  : User interface management functions - Source Build
*******************************************************************************
* History : DD.MM.YYYY Version Description
*         : 07.02.2012 V1.0    First Release (Prototype)
*         : 01.06.2012 V1.1    Updated prototype release to include motor control library
                               plus added build option for RL78G14 64 pin device
*         : 15.06.2012 V2.0    Production release plus rebuild for new IAR EW release
*                              modified file organisation to support object creation
******************************************************************************/


/******************************************************************************
Includes   <System Includes> , "Project Includes"
******************************************************************************/
#include "globdef.h"
#include "userif.h"
#include "motorcontrol.h"

/******************************************************************************
Macro definitions
******************************************************************************/

/* timing defines */
#define	LED_TIME_FUN	20	// 200ms led blinking time
#define	LED_TIME_ALL	3	// 30ms led blinking time

/* serial communication protocol */
#define	N_CENT			0	/* slave identification number */
#define SEL_MAS			'?'	/* master frame identification character (question) */
#define SEL_SLA			'!'	/* slave frame identification character (answer OK) */
#define SEL_NOK			'#'	/* slave frame identification character (answer NOK) */
#define CCO			'c'	/* Check Com operation code */
#define NCO			'C'	/* Check Com operation code (answer) */
#define RWO			'w'	/* Read Word operation code */
#define WWO			'W'	/* Write Word operation code */
#define RW3			'y'	/* Read Word 3 operation code (parameter minimum values) */
#define RW4			'z'	/* Read Word 4 operation code (parameter maximum values) */
#define	LEN_MIN			5	/* shortest frame length, crc included */
#define	LEN_DEL			2	/* additional characters, data excluded, crc included */
#define	IND_LEN			0	/* string length index */
#define	IND_SEL			1	/* string identificator index */
#define	IND_ADS			2	/* station address index */
#define	IND_COD			3	/* operation code index */
#define IND_ADD			4	/* data address index */
#define	IND_ND			5	/* data number index */
#define	IND_ID			6	/* data start index */

/* software related flags */
#define	RX0_ON			(fser0.bit.b0 != 0)
#define	RX0_OFF			(fser0.bit.b0 == 0)
#define	RX0_INI			(fser0.bit.b1 != 0)
#define	TX0_ON			(fser0.bit.b2 != 0)
#define	TX0_OFF			(fser0.bit.b2 == 0)
#define GO_TX0			(fser0.bit.b3 != 0)
#define	SET_RX0_ON		fser0.bit.b0 = 1;
#define	SET_RX0_INI		fser0.bit.b1 = 1;
#define	SET_TX0_ON		fser0.bit.b2 = 1;
#define SET_GO_TX0		fser0.bit.b3 = 1;
#define	RES_RX0_ON		fser0.bit.b0 = 0;
#define	RES_RX0_INI		fser0.bit.b1 = 0;
#define	RES_TX0_ON		fser0.bit.b2 = 0;
#define RES_GO_TX0		fser0.bit.b3 = 0;

// hardware related defines
#define STOP0			ST0L |= 0x01;
#define STOP1			ST0L |= 0x02;
#define STOP_TX_OUT		SOE0L &= 0xFE;
#define START_TX_OUT	        SOE0L |= 0x01;
#define SET_TX_PIN_OUT	        P5 |= 0x02;
#define	SET_SDR00		SDR00 = 0xCE00;
#define	SET_SDR01		SDR01 = 0xCE00;
#define SET_SIR00		SIR00L |= 0x07;
#define SET_FECT01		SIR01L |= 0x04;
#define SET_PEC01		SIR01L |= 0x02;
#define SET_OVC01		SIR01L |= 0x01;
#define SET_SIR01		SIR01L |= 0x07;
#define TSF00			(SSR00L & 0x20)
#define BFF00			(SSR00L & 0x20)
#define TSF01			(SSR01L & 0x20)
#define BFF01			(SSR01L & 0x20)
#define FEF01			(SSR01L & 0x04)
#define PEF01			(SSR01L & 0x02)
#define OVF01			(SSR01L & 0x01)
#define HWF01			(SSR01L & 0x07)
#define START0			SS0L |= 0x01;
#define START1			SS0L |= 0x02;

#define	TX0_EMPTY		(BFF00 == 0)
#define	RX0_FULL		(BFF01 != 0)
#define	RX0_OVR_ERR		(OVR01 != 0)
#define	RX0_FRA_ERR		(FEF01 != 0)
#define	RX0_PAR_ERR		(PEF01 != 0)
#define RX0_HW_ERR		(HWF01 != 0)
#define TX0_END			(TSF00 == 0)

#define	RES_TX0_EMPTY	        // authomatic: not needed
#define	RES_RX0_FULL	        // authomatic: not needed

#define	RES_RX0_OVR_ERR	  SET_OVC01
#define	RES_RX0_FRA_ERR	  SET_FECT01
#define	RES_RX0_PAR_ERR	  SET_PEC01
#define	RES_RX0_HW_ERR	  SET_SIR01

#define INI_UART0   { ST0 = 0x0003; SPS0 = 0x0044; SMR00 = 0x0022; SMR01 = 0x0122; SCR00 = 0x8097; \
  		      SCR01 = 0x4497; SDR00 = 0xCE00; SDR01 = 0xCE00; SOL0 = 0x0000; SO0 = 0x0F0F; \
		      SOE0 = 0x0001; P5 |= 0x02; SIR00 = 0x0007; SIR01 = 0x0007; }

#define	RES_TX0	    { STOP0 SET_TX_PIN_OUT STOP_TX_OUT }
#define	SET_TX0	    { INI_UART0 START0 }

#define	RES_RX0	    STOP1
#define	SET_RX0	    { INI_UART0 START1 }

#define RX0_REG	    RXD0
#define TX0_REG	    TXD0

#define	RX0_TIM	    20	        // rx end delay (1= sampling time)
#define	TX0_TIM	    15	        // tx end delay (1= sampling time)

#define	START_TX0   {SET_TX0              \
		     rtx0_cnt = 0;        \
		     rtx0_tim = TX0_TIM;  \
		     SET_TX0_ON           }

#define	START_RX0   {rtx0_cnt = 0;        \
		     RES_RX0_INI          \
		     SET_RX0		  \
		     SET_RX0_ON		  }

#define	RTX0_LEN    220	        // tx/rx buffer dim

/******************************************************************************
Private global variables
******************************************************************************/

const uint8_t crc_tab[256]={		// DOW CRC calculation table
  0,  94, 188, 226,  97,  63, 221, 131, 194, 156, 126,  32, 163, 253,  31,  65,
157, 195,  33, 127, 252, 162,  64,  30,  95,   1, 227, 189,  62,  96, 130, 220,
 35, 125, 159, 193,  66,  28, 254, 160, 225, 191,  93,   3, 128, 222,  60,  98,
190, 224,   2,  92, 223, 129,  99,  61, 124,  34, 192, 158,  29,  67, 161, 255,
 70,  24, 250, 164,  39, 121, 155, 197, 132, 218,  56, 102, 229, 187,  89,   7,
219, 133, 103,  57, 186, 228,   6,  88,  25,  71, 165, 251, 120,  38, 196, 154,
101,  59, 217, 135,   4,  90, 184, 230, 167, 249,  27,  69, 198, 152, 122,  36,
248, 166,  68,  26, 153, 199,  37, 123,  58, 100, 134, 216,  91,   5, 231, 185,
140, 210,  48, 110, 237, 179,  81,  15,  78,  16, 242, 172,  47, 113, 147, 205,
 17,  79, 173, 243, 112,  46, 204, 146, 211, 141, 111,  49, 178, 236,  14,  80,
175, 241,  19,  77, 206, 144, 114,  44, 109,  51, 209, 143,  12,  82, 176, 238,
 50, 108, 142, 208,  83,  13, 239, 177, 240, 174,  76,  18, 145, 207,  45, 115,
202, 148, 118,  40, 171, 245,  23,  73,   8,  86, 180, 234, 105,  55, 213, 139,
 87,   9, 235, 181,  54, 104, 138, 212, 149, 203,  41, 119, 244, 170,  72,  22,
233, 183,  85,  11, 136, 214,  52, 106,  43, 117, 151, 201,  74,  20, 246, 168,
116,  42, 200, 150,  21,  75, 169, 247, 182, 232,  10,  84, 215, 137, 107,  53};

abyte_t
	fser0,
	fser1;

uint16_t
	rtx0_tim = 0,
	rtx0_cnt = 0,
	rtx0_num = 0,
	mas_ind,		// fifo master index
	sla_ind;		// fifo slave index
uint8_t
	rtx0_buf[RTX0_LEN];	// tx/rx buffer (half duplex)

/******************************************************************************
Exported global variables (to be accessed by other files)
******************************************************************************/

UIF_R_t
	UIF_R;		        // ram table for values to be read by PC

UIF_W_t
	UIF_W;		        // ram table for values to be written by PC



/******************************************************************************
Private functions
******************************************************************************/


/******************************************************************************
* Function Name: MC_ReadButtons
* Description  : Buttons reading and debounce
* Arguments    : none
* Return Value : none
******************************************************************************/
void MC_ReadButtons(void)
{
	static uint8_t	but_mem = 0x3C;
	uint8_t			but_in;

	/* buttons sampling */
	but_in = P5 & 0x3C;	// selects bit 2, 3, 4, 5

	/* simple debounce */
	if ( but_in == but_mem )
		buttons.byte=but_in;
	else
		but_mem=but_in;

}	/* end of function MC_ReadButtons() */


/******************************************************************************
* Function Name: b2w
* Description  : 2 bytes vector conversion in a word
* Arguments    : two bytes vector *v
* Return Value : the short obtained by the union of the two bytes
******************************************************************************/
static uint16_t b2w(uint8_t *v)
{
	aword_t	w;

	w.byte.h = v[0];
	w.byte.l = v[1];
	return (w.word);

}	/* end of function b2w(...) */

/******************************************************************************
* Function Name: w2b
* Description  : 1 word conversion in a 2 bytes vector
* Arguments    : the word to convert and the address of the result vector
* Return Value : the address of the result vector
* Note         : it modifies the vector content
******************************************************************************/
static uint8_t *w2b(uint16_t w, uint8_t *v)
{
	aword_t	x;

	x.word = w;
	v[0] = x.byte.h;
	v[1] = x.byte.l;
	return (v);

}	/* end of function w2b(...) */

/******************************************************************************
* Function Name: cal_crc
* Description  : DOW CRC calculation
* Arguments    : the number of bytes to be taken into account and the vector
*                containing these bytes
* Return Value : CRC
* Note         : it uses the DOW CRC calculation table defined before
******************************************************************************/
static uint8_t cal_crc(uint8_t n, uint8_t *v)
{
	uint8_t	cks = 0, i;

	for (i = 0; i < n; i++)
	{
		cks = crc_tab[cks^v[i]];
	}
	return (cks);

}	/* end of function cal_crc(...) */


/******************************************************************************
Exported global functions (to be accessed by other files)
******************************************************************************/

/******************************************************************************
* Function Name: ini_232_pc
* Description  : serial communication init
* Arguments    : none
* Return Value : none
******************************************************************************/
void ini_232_pc(void)
{  
	SAU0EN=1U;		// PER0.2	1-> supply input clock to Serial Array 0 peripheral

	/* the following instructions introduce enought delay for clock activation */
	for (rtx0_cnt = 0; rtx0_cnt < RTX0_LEN; rtx0_cnt++)
	{
		rtx0_buf[rtx0_cnt] = 0;
	}
	rtx0_cnt = 0;
	rtx0_tim = 0,
	rtx0_num = 0;

	RES_GO_TX0
	START_RX0		// sw RX status

}	/* end of function ini_232_pc() */

/******************************************************************************
* Function Name: poll_232_pc
* Description  : serial communication polling
* Arguments    : none
* Return Value : none
* Note         : to be called in the control interrupt
******************************************************************************/
void poll_232_pc(void)
{

	if (TX0_ON)
	{
		if (TX0_EMPTY)					//	if buffer empty
		{
			if (rtx0_cnt < rtx0_num)		//	... if data not finished ...
			{
				RES_TX0_EMPTY			//	... empty buffer flag reset ...
				TX0_REG = rtx0_buf[rtx0_cnt++];	//	... buffer loading
			}
			else if (rtx0_tim)
			{
				--rtx0_tim;
			}
		}
                
		if ((TX0_END) && (rtx0_tim == 0))		//	if tx end ...
		{
			RES_TX0					//	... tx disable ...
			RES_TX0_ON				//	... end tx ...
			START_RX0				//	... rx enable
		}
	}
	else if (RX0_ON)
	{
		if (RX0_HW_ERR)					//	if errors ...
		{
			RES_RX0_HW_ERR				//	... errors reset ...
			RES_RX0					//	... rx disable ...
			START_RX0				//	... start again
		}
		else if (RX0_FULL)				//	if data received ...
		{
			RES_RX0_FULL				//	... flag reset ...
			SET_RX0_INI							//	... rx begun flag setting ...
			rtx0_tim = RX0_TIM;			//	... delay timer reset ...
			if (rtx0_cnt < RTX0_LEN)		//	... if enough space ...
			{
				rtx0_buf[rtx0_cnt++] = RX0_REG;	//		... received data saving ...
			}
			else					//	... if no space ...
			{
				RES_RX0				//		... rx disable ...
				RES_RX0_ON			//		... rx end
			}
		}
		else if (RX0_INI)				//	if rx begun ...
		{
			if (rtx0_tim > 0)
			{					//	... if still time ...
				--rtx0_tim;			//		... wait
			}
			else					//	... if delay elapsed ...
			{
				RES_RX0				//		... rx disable ...
				RES_RX0_ON			//		... end rx
			}
		}
	}

}	/* end of function poll_232_pc() */

/******************************************************************************
* Function Name: ges_232_pc
* Description  : serial communication management
* Arguments    : none
* Return Value : none
* Note         : to be called in the main loop;
*                the slave is always waiting for commands (reception)
*                when a commend is received, it is decoded and, if it make sense,
*                executed; if the command is correctly received or if the errors
*                are in the data or in the addresses, the slave send an answer.
******************************************************************************/
void ges_232_pc(void)
{
	int16_t		add, nda, i;
	uint16_t	add_par, dta_par;

	if (GO_TX0)
	{
		RES_GO_TX0
		START_TX0
	}
	else if (RX0_INI && RX0_OFF)		                // reception ended
	{
		RES_RX0_INI					// reset rx begun flag

		/* no answer situations */
		rtx0_num = rtx0_buf[IND_LEN];
                
		if ((rtx0_num < LEN_MIN) || (rtx0_num > rtx0_cnt) || (rtx0_buf[IND_SEL] != SEL_MAS) || (rtx0_buf[IND_ADS] != N_CENT))
		{
			START_RX0
			return;
		}
                
		if (rtx0_buf[rtx0_num - 1] != cal_crc((rtx0_num - 1), rtx0_buf))
		{
			START_RX0
			return;
		}

		/* operation codes decoding and answer building */
		rtx0_buf[IND_SEL] = SEL_SLA;
    	        rtx0_num = LEN_MIN;				// frame length
                
		switch(rtx0_buf[IND_COD])
		{
			case CCO:	                        // check
				rtx0_buf[IND_COD] = NCO;
				break;
                                
			case RWO:
				add = rtx0_buf[IND_ADD];
				nda = rtx0_buf[IND_ND];
                                
				if (add < NUM_PAR)		// reading from parameters table
				{
					if ((nda > NUM_PAR) || (add > (NUM_PAR - nda)))
					{
						rtx0_buf[IND_SEL] = SEL_NOK;
						break;
					}
                                        
					for (i = 0; i < nda; i++)
					{
						w2b(par_vec[add + i], &(rtx0_buf[IND_ID + (i << 1)]));
					}
                                        
				    rtx0_num += LEN_DEL + (nda << 1);   // frame length
				}
				else				// reading from UIF data structure
				{
					add -= NUM_PAR;
                                        
					if ((nda > N_RAM_READ) || (add > (N_RAM_READ - nda)))
					{
						rtx0_buf[IND_SEL] = SEL_NOK;
						break;
					}
                                        
					for (i = 0; i < nda; i++)
					{
						w2b(UIF_R.ram_tab[add + i], &(rtx0_buf[IND_ID + (i << 1)]));
					}
                                        
				    rtx0_num += LEN_DEL + (nda << 1);   // frame length
				}
				break;
                                
			case WWO:
				add = rtx0_buf[IND_ADD];
				nda = rtx0_buf[IND_ND];
                                
				if (add < NUM_PAR)		// writing in parameters table
				{
					if ((nda > NUM_PAR) || (add > (NUM_PAR - nda)))
					{
						rtx0_buf[IND_SEL] = SEL_NOK;
						break;
					}
                                        
					for (i = 0; i < nda; i++)
					{
						dta_par = b2w(&(rtx0_buf[IND_ID + (i << 1)]));
						add_par = add + i;
						req_eqp(dta_par, add_par);
					}
				}
				else				// writing in UIF data structure
				{
					add -= NUM_PAR;
					if ((nda > N_RAM_WRITE) || (add > (N_RAM_WRITE - nda)))
					{
						rtx0_buf[IND_SEL] = SEL_NOK;
						break;
					}
					for (i = 0; i < nda; i++)
					{
						UIF_W.ram_tab[add + i]=b2w(&(rtx0_buf[IND_ID + (i << 1)]));
					}
				}
				break;
                                
			case RW3:	                        // parameter minimum values reading
				add = rtx0_buf[IND_ADD];
				nda = rtx0_buf[IND_ND];
                                
				if ((add >= NUM_PAR) || (nda > (NUM_PAR - add)))
				{
					rtx0_buf[IND_SEL] = SEL_NOK;
					break;
				}
                                
				for (i = 0; i < nda; i++)
				{
					w2b(par_min[add + i], &(rtx0_buf[IND_ID + (i << 1)]));
				}
                                
			        rtx0_num += LEN_DEL;
			        rtx0_num += nda << 1;
				break;
                                
			case RW4:	                          // parameter maximum values reading
				add = rtx0_buf[IND_ADD];
				nda = rtx0_buf[IND_ND];
                                
				if ((add >= NUM_PAR) || (nda > (NUM_PAR - add)))
				{
					rtx0_buf[IND_SEL] = SEL_NOK;
					break;
				}
                                
				for (i = 0; i < nda; i++)
				{
					w2b(par_max[add + i], &(rtx0_buf[IND_ID + (i << 1)]));
				}
                                
			        rtx0_num += LEN_DEL;
			        rtx0_num += nda << 1;
				break;
                                
			default:
				rtx0_buf[IND_SEL] = SEL_NOK;
				break;
		}

		/* crc calculation and answer start transmission */
		rtx0_buf[IND_LEN] = rtx0_num;
		rtx0_buf[rtx0_num - 1] = cal_crc(rtx0_num - 1, rtx0_buf);
		rtx0_buf[rtx0_num] = 0;
		SET_GO_TX0
		return;
    }

}	/* end of function ges_232_pc() */

/******************************************************************************
* Function Name: ges_led
* Description  : led blinking management
* Arguments    : none
* Return Value : none
* Note         : to be called in the main loop
******************************************************************************/
void ges_led(void)
{
	static uint16_t led_cnt;

	switch(UIF_R.var.all)
	{
		case 0:
			UIF_R.var.flg &= WCLR7;
			SET_LD2_OFF
                          
			if (led_cnt < LED_TIME_FUN)
				++led_cnt;
			else
			{
				led_cnt = 0;
				if (LD1_OFF)
				{
					SET_LD1_ON
				}
				else
				{
					SET_LD1_OFF
				}
			}
			break;
                        
		default:
			UIF_R.var.flg |= WSET7;
			SET_LD1_OFF
                          
			if (led_cnt < LED_TIME_ALL)
			{
				++led_cnt;
			}
			else
			{
				led_cnt = 0;
				if (LD2_OFF)
				{
					SET_LD2_ON
				}
				else
				{
					SET_LD2_OFF
				}
			}
			break;
	}

}	/* end of function ges_led() */
